#! /usr/bin/env sh
show_networks() {
  # No point in looking at lan bridge or loopback devices.
  # We don't want to put a default route on loopback,
  # and a lan bridge device isn't useful for bonding.
  # Other lanN ports might be useful, because they may
  # be reassigned to wan ports. (If so, they will show up
  # in this list, if not, they'll be in the lan bridge.)
  # The wan bridge might also be useful for bonding, esp.
  # in the case that it wasn't reconfigured as a "standalone"
  # non-bridge network device.
  uci show network | grep device= | grep -vE 'br-lan|loopback' | sed 's/\.device=.*//'
}
show_metrics() {
  show_networks | while read nw ; do uci show "$nw.metric" 2>/dev/null || echo "$nw.metric=missing" ; done
}
set_metrics() {
  if echo "$*" | grep -q -- --non-interactive
  then
    # Best guess, without user interaction.
    # Special cases for the wan-looking devices, and other that don't look like wans.
    # In the case that there is are several non-lan-bridge networks, which should have a higher metric.
    show_metrics | grep -vE '\..+?wan.+?\.metric=' | awk -F'[.=]' '{ metric = 1000 + 10 * NR; printf "%s.%s.%s='\''%d'\''\n", $1, $2, $3, metric; }' | xargs -n 1 uci set
    show_metrics | grep -E '\..+?wan.+?\.metric=' | awk -F'[.=]' '{ metric = 10 * NR; printf "%s.%s.%s='\''%d'\''\n", $1, $2, $3, metric; }' | xargs -n 1 uci set
    uci commit
  else
    set_metrics_interactive_snippet
    trap 'rm -f /tmp/bonding-support-metrics' EXIT
    show_networks | while read nw
    do
      echo "Metric for $nw (Enter a number, or q to quit)"
      read -r metric < /dev/tty
      [ "$metric" = q ] && exit 1
      [ -z "$metric" ] && echo "Empty metric." && exit 1
      ! [ "$metric" -eq "$metric" ] 2>/dev/null && echo "Not a number." && exit 1
      show_metrics | grep -q "='$metric'" && echo "Metric $metric already in use and must be unique." && exit 1
      echo "set $nw.metric=$metric" >> /tmp/bonding-support-metrics
    done || exit 1
    echo "Metrics to be set:"
    cat /tmp/bonding-support-metrics
    read -r -p "Commit these changes? [Enter/y/N] " response < /dev/tty
    if [ "$response" = y ] || [ "$response" = Y ] || [ -z "$response" ]
    then
      uci batch < /tmp/bonding-support-metrics
      uci commit
      echo "Changes committed."
    else
      echo "Changes not committed."
      exit 1
    fi
  fi
}
set_metrics_interactive_snippet() {
  cat <<EOF
This will set the metric for each network device, either interactively or non-interactively.

For Speedify's bonding to work properly, each network device must have a unique metric.
These will be set interactively unless --non-interactive is specified, in which case the
script will make a best guess, preferring WAN-looking networks over others.

If Speedify is off, when a route matches several networks, and each route is equally specific,
the lower metric wins. It's a tie-breaker.
If Speedify is on, network conditions are determined on-the-fly.

The lowest metric should be your most preferred network.
All metrics should be unique.
EOF
}
usage() {
  cat <<EOF
$0

  set-metrics [ --non-interactive ]
$(set_metrics_interactive_snippet | sed 's/^/    /')

  show-networks
    List all non-lan-bridge, non-loopback network devices.

  show-metrics
    List the metric for each network device from show-networks.

  help, --help, -h
    Show this help message.
EOF
}
case "$1" in
  show-networks)
    show_networks
    ;;
  show-metrics)
    show_metrics
    ;;
  set-metrics)
    set_metrics "$@"
    ;;
  help|--help|-h)
    usage
    ;;
  *)
    usage
    exit 1
    ;;
esac
